from django.contrib.staticfiles.handlers import StaticFilesHandler
from .views import serve


class AssetFilesHandler(StaticFilesHandler):

    def serve(self, request):
        return serve(request, self.file_path(request.path), insecure=True)



import warnings

from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module

from gears.asset_handler import BaseAssetHandler
from gears.finders import BaseFinder


_cache = {}


def _get_module(path):
    try:
        return import_module(path)
    except ImportError, e:
        raise ImproperlyConfigured('Error importing module %s: "%s".' % (path, e))


def _get_module_attr(module_path, name):
    try:
        return getattr(_get_module(module_path), name)
    except AttributeError:
        raise ImproperlyConfigured('Module "%s" does not define a "%s" obj.' % (module_path, name))


def _get_object(path):
    if path not in _cache:
        _cache[path] = _get_module_attr(*path.rsplit('.', 1))
    return _cache[path]


def get_cache(path, options=None):
    cls = _get_object(path)
    return cls(**(options or {}))


def get_finder(path, options=None):
    cls = _get_object(path)
    if not issubclass(cls, BaseFinder):
        raise ImproperlyConfigured('"%s" is not a subclass of BaseFinder.' % path)
    return cls(**(options or {}))


def get_asset_handler(path, options=None):
    obj = _get_object(path)
    try:
        if issubclass(obj, BaseAssetHandler):
            return obj.as_handler(**(options or {}))
    except TypeError:
        pass
    if callable(obj):
        if options is not None:
            warnings.warn('%r is provided as %r handler options, but not used '
                'because this handler is not a BaseAssethandler subclass.'
                % (options, path))
        return obj
    raise ImproperlyConfigured('"%s" must be a BaseAssetHandler subclass or callable object' % path)



from optparse import make_option

from django.conf import settings
from django.contrib.staticfiles.handlers import StaticFilesHandler
from django.core.management.commands.runserver import BaseRunserverCommand

from django_gears.handlers import AssetFilesHandler


class Command(BaseRunserverCommand):

    option_list = BaseRunserverCommand.option_list + (
        make_option('--noassets', action='store_false',
                    dest='use_assets_handler', default=True,
                    help='Tells Django to NOT automatically serve asset files '
                         'at STATIC_URL.'),
        make_option('--nostatic', action='store_false',
                    dest='use_static_handler', default=True,
                    help='Tells Django to NOT automatically serve static '
                         'files at STATIC_URL.'),
        make_option('--insecure', action='store_true', dest='insecure_serving',
                    default=False, help='Allows serving static files even if '
                                        'DEBUG is False'),
    )
    help = ('Starts a lightweight Web server for development and also serves '
            'static files.')

    def get_handler(self, *args, **options):
        handler = super(Command, self).get_handler(*args, **options)
        use_assets_handler = options.get('use_assets_handler', True)
        use_static_handler = options.get('use_static_handler', True)
        insecure_serving = options.get('insecure_serving', False)
        if use_static_handler and (settings.DEBUG or insecure_serving):
            if use_assets_handler:
                return AssetFilesHandler(handler)
            return StaticFilesHandler(handler)
        return handler



from django.conf import settings
from gears.environment import Environment, DEFAULT_PUBLIC_ASSETS
from .utils import get_cache, get_finder, get_asset_handler


DEFAULT_CACHE = 'gears.cache.SimpleCache'

DEFAULT_FINDERS = (
    ('gears.finders.FileSystemFinder', {
        'directories': getattr(settings, 'GEARS_DIRS', ()),
    }),
)

DEFAULT_MIMETYPES = {
    '.css': 'text/css',
    '.js': 'application/javascript',
}

DEFAULT_PREPROCESSORS = {
    'text/css': 'gears.processors.DirectivesProcessor',
    'application/javascript': 'gears.processors.DirectivesProcessor',
}

DEFAULT_POSTPROCESSORS = {
    'text/css': 'gears.processors.HexdigestPathsProcessor',
}

GEARS_DEBUG = getattr(settings, 'GEARS_DEBUG', settings.DEBUG)

GEARS_URL = getattr(settings, 'GEARS_URL', settings.STATIC_URL)


path = getattr(settings, 'GEARS_CACHE', DEFAULT_CACHE)
if isinstance(path, (list, tuple)):
    path, options = path
else:
    options = None
cache = get_cache(path, options)

environment = Environment(
    root=getattr(settings, 'GEARS_ROOT'),
    public_assets=getattr(settings, 'GEARS_PUBLIC_ASSETS', DEFAULT_PUBLIC_ASSETS),
    cache=cache,
    gzip=getattr(settings, 'GEARS_GZIP', False),
)

for path in getattr(settings, 'GEARS_FINDERS', DEFAULT_FINDERS):
    if isinstance(path, (list, tuple)):
        path, options = path
    else:
        options = None
    environment.finders.register(get_finder(path, options))

mimetypes = getattr(settings, 'GEARS_MIMETYPES', DEFAULT_MIMETYPES)
for extension, mimetype in mimetypes.items():
    environment.mimetypes.register(extension, mimetype)

for extension, path in getattr(settings, 'GEARS_COMPILERS', {}).items():
    if isinstance(path, (list, tuple)):
        path, options = path
    else:
        options = {}
    environment.compilers.register(extension, get_asset_handler(path, options))

preprocessors = getattr(settings, 'GEARS_PREPROCESSORS', DEFAULT_PREPROCESSORS)
for mimetype, paths in preprocessors.items():
    if not isinstance(paths, (list, tuple)):
        paths = [paths]
    for path in paths:
        environment.preprocessors.register(mimetype, get_asset_handler(path))

postprocessors = getattr(settings, 'GEARS_POSTPROCESSORS', DEFAULT_POSTPROCESSORS)
for mimetype, paths in postprocessors.items():
    if not isinstance(paths, (list, tuple)):
        paths = [paths]
    for path in paths:
        environment.postprocessors.register(mimetype, get_asset_handler(path))

compressors = getattr(settings, 'GEARS_COMPRESSORS', {})
for mimetype, path in compressors.items():
    environment.compressors.register(mimetype, get_asset_handler(path))



import mimetypes
import posixpath
import time
import urllib

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.contrib.staticfiles.views import serve as staticfiles_serve
from django.http import HttpResponse
from django.utils.http import http_date

from gears.assets import build_asset
from gears.compat import bytes
from gears.exceptions import FileNotFound

from .settings import environment

MAX_AGE = 60 * 60 * 24 * 7 # 1 week


def serve(request, path, **kwargs):
    if not settings.DEBUG and not kwargs.get('insecure'):
        raise ImproperlyConfigured(
            "The gears view can only be used in debug mode or if the "
            "--insecure option of 'runserver' is used.")

    # It is only required check because we generate
    # version arg for each file
    if 'HTTP_IF_MODIFIED_SINCE' in request.META:
        response = HttpResponse()
        response['Expires'] = http_date(time.time() + MAX_AGE)
        response.status_code = 304
        return response

    normalized_path = posixpath.normpath(urllib.unquote(path)).lstrip('/')
    try:
        asset = build_asset(environment, normalized_path)
    except FileNotFound:
        return staticfiles_serve(request, path, **kwargs)

    last_modified = asset.mtime
    if request.GET.get('body'):
        asset = asset.processed_source
    mimetype, encoding = mimetypes.guess_type(normalized_path)
    mimetype = mimetype or 'application/octet-stream'
    response = HttpResponse(bytes(asset), mimetype=mimetype)
    if encoding:
        response['Content-Encoding'] = encoding
    response['Last-Modified'] = http_date(last_modified)
    return response



from __future__ import absolute_import
from django.template import Node, Library, TemplateSyntaxError
from gears.assets import build_asset
from ..settings import environment, GEARS_URL, GEARS_DEBUG


register = Library()


class AssetTagNode(Node):

    def __init__(self, logical_path, debug):
        self.logical_path = logical_path
        self.debug = debug

    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        if len(bits) not in (2, 3):
            msg = '%r tag takes one argument: the logical path to the public asset'
            raise TemplateSyntaxError(msg % bits[0])
        debug = (len(bits) == 3)
        if debug and bits[2] != 'debug':
            msg = "Second (optional) argument to %r tag must be 'debug'"
            raise TemplateSyntaxError(msg % bits[0])
        logical_path = parser.compile_filter(bits[1])
        return cls(logical_path, debug)


    def render(self, context):
        logical_path = self.logical_path.resolve(context)
        if self.debug or GEARS_DEBUG:
            asset = build_asset(environment, logical_path)
            paths = (('%s?body=1&v=%s' % (r.attributes.logical_path, r.mtime))\
                for r in asset.requirements)
        else:
            if logical_path in environment.manifest.files:
                logical_path = environment.manifest.files[logical_path]
            paths = (logical_path,)
        return '\n'.join((self.template % path) for path in paths)


class CSSAssetTagNode(AssetTagNode):

    template = u'<link rel="stylesheet" href="%s%%s">' % GEARS_URL


class JSAssetTagNode(AssetTagNode):

    template = u'<script src="%s%%s"></script>' % GEARS_URL


@register.tag
def css_asset_tag(parser, token):
    return CSSAssetTagNode.handle_token(parser, token)


@register.tag
def js_asset_tag(parser, token):
    return JSAssetTagNode.handle_token(parser, token)



from django.core.management.base import NoArgsCommand
from django_gears.settings import environment


class Command(NoArgsCommand):

    help = 'Collect assets in a single location.'

    def handle_noargs(self, **options):
        environment.save()
